//global variables
var mTipbar = null;

//constants
const HTML_NS = "http://www.w3.org/1999/xhtml";

/*********************************************************************************
 *  function:     onTipbarLoad
 *	description:  window load function
 ********************************************************************************/
function onTipbarLoad() 
{  
  //create tipbar object
  mTipbar = new tipbar();

  //see if db is stored on hidden dom window
  var hdw = "@mozilla.org/appshell/appShellService;1";
  hdw = Components.classes[hdw].getService(Components.interfaces.nsIAppShellService)
  hdw = hdw.hiddenDOMWindow
  
  //load db from hidden window or parse from XML
  if (hdw.db)
    mTipbar.setDb(hdw.db);
  else
  {
    if (mTipbar.prefs["offline"].value) 
      getXML(mTipbar.prefs["local"].value);
    else
      getXML(mTipbar.prefs["remote"].value);
  }
}

/*********************************************************************************
 *  function:     onTipbarLoad
 *	description:  window unload function
 ********************************************************************************/
function onTipbarUnload() 
{   
  //destroy global objects
  mTipbar = null;
}

/*********************************************************************************
 *  function:     getXML
 *	description:  used to retrieve tips from XML file
 ********************************************************************************/
function getXML(aURL)
{
    mTipbar.setState("loading");
  
  try {        
    //create request and send
    var req = new XMLHttpRequest();
    req.open("GET",aURL,true);
    req.onload = function() 
    { 
      XMLResponse(aURL, req.responseXML, req.status);
    }
    req.send(null);
  }
  catch(e)
  {
    mTipbar.setState("error");
  }
}

/*********************************************************************************
 *  function:     XMLResponse
 *	description:  called ASYNC from getXML function
 ********************************************************************************/
function XMLResponse(aURL, aXMLDoc, aStatus)
{
  if (!aXMLDoc || aStatus > 200)
  {
    if (aURL != mTipbar.prefs["remote"].value)
      mTipbar.setState("error");
    else
    {
      if (!mTipbar.prefs["offline"].value)
        getXML(mTipbar.prefs["local"].value);
      else
        mTipbar.setState("error");
    }
  }
  else
  {
    var tipNodes = aXMLDoc.getElementsByTagName("tip");
    var tmpDb = new Array();
    var count = 0;
    var newContent, walker, appNodes;
    for (var i = 0; i < tipNodes.length; ++i)
    {
      appNodes = tipNodes[i].getElementsByTagName("app");
      for (var x = 0; x < appNodes.length; ++x)
      {
        if (appNodes[x].getAttribute("id") == mTipbar.prefs["app"].value)
        {
          newContent = document.createElementNS(HTML_NS,"html:div");
          walker = document.createTreeWalker(tipNodes[i].getElementsByTagName("content")[0], NodeFilter.SHOW_ALL, null, true);
          walkContent(walker,newContent);
          tmpDb[count] = new tip(tipNodes[i].getAttribute("id"),newContent);
          ++count
          break;
        }
      }
    }  
    if (tmpDb[0])
    {
      //store the db on the hidden dom window
      var hdw = "@mozilla.org/appshell/appShellService;1";
      hdw = Components.classes[hdw].getService(Components.interfaces.nsIAppShellService)
      hdw = hdw.hiddenDOMWindow    
      hdw.db = tmpDb;
      //set database
      mTipbar.setDb(tmpDb);  
    }
    else
      mTipbar.setState("error");
  }
}

/*********************************************************************************
 *  function:     walkContent
 *	description:  used to parse the tip content into an HTML fragment
 ********************************************************************************/
function walkContent(walker,destNode)
{
  // walk the TreeWalker tree...
  if(walker.firstChild()) 
  {
    do {
      // ...and create an output node for every node we find
      var node, newNode;
      node = walker.currentNode;
      if (node.nodeType == Node.ELEMENT_NODE)
      {
        switch (node.nodeName) 
        {
          case "click":
            newNode = document.createElementNS(HTML_NS, "html:b");
            break;
          case "key":
            newNode = document.createElementNS(HTML_NS, "html:i");
            break;
          case "img":
            newNode = document.createElementNS(HTML_NS, "html:img");
            newNode.setAttribute("src",node.getAttribute("src"));
            break;
          case "text":
            newNode = document.createElementNS(HTML_NS, "html:a");
            newNode.setAttribute("href",node.getAttribute("string"));
            newNode.setAttribute("target", "_blank");
            newNode.setAttribute("style","text-decoration: underline;");
            break;
          default:
            newNode = document.createElementNS(HTML_NS, "html:b");
            break;
        }
      }
      else
      {
        newNode = document.createTextNode(node.nodeValue);
      }
          
      // insert the output node and recursivly walk the children
      // of every node
      destNode.appendChild(newNode);
      walkContent(walker, newNode);

    } while(walker.nextSibling());

    // don't forget to return the treewalker to it's previous state
    // before exiting the function
    walker.parentNode();
  }
}

/*********************************************************************************
 *  function:     tipbar
 *	description:  main controlling object
 ********************************************************************************/
function tipbar()
{
  this.init();
}

tipbar.prototype.init = function()
{
  //create display object
  this.display = new display();
  
  //get preference branch
  var prefService = "@mozilla.org/preferences-service;1";
  prefService = Components.classes[prefService].getService(Components.interfaces.nsIPrefService); 
  this.branch = prefService.getBranch(null);
  
  //set default prefs
  this.prefs = new Object();
  this.prefs["app"] = new pref("general.useragent.vendor","char","Mozilla",false);
  this.prefs["offline"] = new pref("browser.offline","bool",false,false);
  this.prefs["current"] = new pref("tipbar.current","int",0,true);
  this.prefs["remote"] = new pref("tipbar.url.remote","complex","http://downloads.mozdev.org/tipbar/tips.xml",true);
  this.prefs["local"] = new pref("tipbar.url.local","complex","chrome://tipbar/content/tips.xml",true);

  //get actual preferenc values
  for (var i in this.prefs) 
    this.prefs[i].get(this.branch);

  //setup null db
  this.db = null;
  this.length = 0; 
} 

tipbar.prototype.setState = function(aState)
{
  this.display.setState(aState);
}

tipbar.prototype.setDb = function(aDb)
{
  this.db = aDb;
  this.length = this.db.length-1;
  this.setState("loaded");
  this.showTip("next");
}

tipbar.prototype.showTip = function(aDirection)
{
  if (this.db)
  {
    switch (aDirection)
    {
      case "previous":
        if (this.prefs["current"].value <= 0)
          this.prefs["current"].value = this.length;
        else
          --this.prefs["current"].value; 
        break;
      case "next":
      default:
        if (this.prefs["current"].value >= this.length)
          this.prefs["current"].value = 0;
        else
          ++this.prefs["current"].value;
        break;
    }
    //save current to pref and display
    this.prefs["current"].set(this.branch);
    this.display.setTip(this.db[this.prefs["current"].value]);
  }
}

/*********************************************************************************
 *  function:     display
 *	description:  object used to handle all display requests
 ********************************************************************************/
function display()
{
  this.init();
}

display.prototype.init =  function()
{
  this.header = document.getElementById("tipbar-header");
  this.body = document.getElementById("tipbar-body");
  this.next = document.getElementById("tipbar-next");
  this.prev = document.getElementById("tipbar-prev");
  this.setState("ready");
}

display.prototype.setState = function(aState)
{
  this.removeTip();
  this.header.setAttribute("value",this.header.getAttribute(aState));
  this.next.removeAttribute("disabled");
  this.prev.removeAttribute("disabled");
  
  if (aState != "loaded")
  {
    this.next.setAttribute("disabled","true");
    this.prev.setAttribute("disabled","true");      
  }
}
  
display.prototype.removeTip = function()
{
  while (this.body.hasChildNodes())
    this.body.removeChild(this.body.lastChild);
}
  
display.prototype.setTip = function(aTip)
{
  this.removeTip();
  this.body.appendChild(aTip.content);
}

/*********************************************************************************
 *  function:     tip
 *	description:  tip object
 ********************************************************************************/
function tip (aId, aContent)
{
  this.init(aId, aContent);
}

tip.prototype.init = function(aId, aContent)
{
  this.id = aId;
  this.content = aContent;
}

/*********************************************************************************
 *  function:     pref
 *	description:  object used to handle preferences
 ********************************************************************************/
function pref(aName, aType, aValue, aOwned)
{
  this.init(aName, aType, aValue, aOwned);
}

pref.prototype.init = function(aName, aType, aValue, aOwned)
{
  //this will setup preferences with default values
  this.name = aName;
  this.type = aType;
  this.value = aValue;
  this.owned = aOwned;
}

pref.prototype.get = function(aBranch)
{
  try 
  {
    switch (this.type)
    {
      case "bool":
        this.value = aBranch.getBoolPref(this.name);
        break;
      case "int":
        this.value = aBranch.getIntPref(this.name);
        break;
      case "char":
        this.value = aBranch.getCharPref(this.name);
        break;
      case "complex":
      default:
        this.value = aBranch.getComplexValue(this.name, Components.interfaces.nsIPrefLocalizedString).data;
        break;
    }
  }
  catch(e)
  {
    this.set(aBranch);
  }
}


pref.prototype.set = function(aBranch)
{
  if (this.owned)
  {
    try 
    {
      switch (this.type)
      {
        case "bool":
          aBranch.setBoolPref(this.name,this.value);
          break;
        case "int":
          aBranch.setIntPref(this.name,this.value);
          break;
        case "char":
          aBranch.setCharPref(this.name,this.value);
          break;
        case "complex":
        default:
          var str = "@mozilla.org/supports-string;1";
          str = Components.classes[str].createInstance(Components.interfaces.nsISupportsString);
          str.data = this.value;
          aBranch.setComplexValue(this.name, Components.interfaces.nsISupportsString, str);
          break;
      }
    }
    catch(e) {}
  }
} 
 